# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .quantity_rounding import QuantityRounding as QuantityRounding
from .product_list_params import ProductListParams as ProductListParams
from .quantity_conversion import QuantityConversion as QuantityConversion
from .product_create_params import ProductCreateParams as ProductCreateParams
from .product_list_response import ProductListResponse as ProductListResponse
from .product_update_params import ProductUpdateParams as ProductUpdateParams
from .rate_card_list_params import RateCardListParams as RateCardListParams
from .product_archive_params import ProductArchiveParams as ProductArchiveParams
from .product_create_response import ProductCreateResponse as ProductCreateResponse
from .product_list_item_state import ProductListItemState as ProductListItemState
from .product_retrieve_params import ProductRetrieveParams as ProductRetrieveParams
from .product_update_response import ProductUpdateResponse as ProductUpdateResponse
from .quantity_rounding_param import QuantityRoundingParam as QuantityRoundingParam
from .rate_card_create_params import RateCardCreateParams as RateCardCreateParams
from .rate_card_list_response import RateCardListResponse as RateCardListResponse
from .rate_card_update_params import RateCardUpdateParams as RateCardUpdateParams
from .product_archive_response import ProductArchiveResponse as ProductArchiveResponse
from .rate_card_archive_params import RateCardArchiveParams as RateCardArchiveParams
from .product_retrieve_response import ProductRetrieveResponse as ProductRetrieveResponse
from .quantity_conversion_param import QuantityConversionParam as QuantityConversionParam
from .rate_card_create_response import RateCardCreateResponse as RateCardCreateResponse
from .rate_card_retrieve_params import RateCardRetrieveParams as RateCardRetrieveParams
from .rate_card_update_response import RateCardUpdateResponse as RateCardUpdateResponse
from .rate_card_archive_response import RateCardArchiveResponse as RateCardArchiveResponse
from .rate_card_retrieve_response import RateCardRetrieveResponse as RateCardRetrieveResponse
from .named_schedule_update_params import NamedScheduleUpdateParams as NamedScheduleUpdateParams
from .named_schedule_retrieve_params import NamedScheduleRetrieveParams as NamedScheduleRetrieveParams
from .named_schedule_retrieve_response import NamedScheduleRetrieveResponse as NamedScheduleRetrieveResponse
from .rate_card_retrieve_rate_schedule_params import (
    RateCardRetrieveRateScheduleParams as RateCardRetrieveRateScheduleParams,
)
from .rate_card_retrieve_rate_schedule_response import (
    RateCardRetrieveRateScheduleResponse as RateCardRetrieveRateScheduleResponse,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "ContractCreateHistoricalInvoicesParams",
    "Invoice",
    "InvoiceUsageLineItem",
    "InvoiceUsageLineItemSubtotalsWithQuantity",
]


class ContractCreateHistoricalInvoicesParams(TypedDict, total=False):
    invoices: Required[Iterable[Invoice]]

    preview: Required[bool]


class InvoiceUsageLineItemSubtotalsWithQuantity(TypedDict, total=False):
    exclusive_end_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    inclusive_start_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    quantity: Required[float]


class InvoiceUsageLineItem(TypedDict, total=False):
    exclusive_end_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    inclusive_start_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    product_id: Required[str]

    presentation_group_values: Dict[str, str]

    pricing_group_values: Dict[str, str]

    quantity: float

    subtotals_with_quantity: Iterable[InvoiceUsageLineItemSubtotalsWithQuantity]


class Invoice(TypedDict, total=False):
    contract_id: Required[str]

    credit_type_id: Required[str]

    customer_id: Required[str]

    exclusive_end_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    inclusive_start_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    issue_date: Required[Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]]

    usage_line_items: Required[Iterable[InvoiceUsageLineItem]]

    billable_status: Literal["billable", "unbillable"]
    """This field's availability is dependent on your client's configuration."""

    breakdown_granularity: Literal["HOUR", "DAY"]

    custom_fields: Dict[str, str]
    """Custom fields to be added eg. { "key1": "value1", "key2": "value2" }"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tier import Tier as Tier
from .commit_rate import CommitRate as CommitRate
from .property_filter import PropertyFilter as PropertyFilter
from .base_usage_filter import BaseUsageFilter as BaseUsageFilter
from .event_type_filter import EventTypeFilter as EventTypeFilter
from .payment_gate_config import PaymentGateConfig as PaymentGateConfig
from .base_threshold_commit import BaseThresholdCommit as BaseThresholdCommit
from .commit_specifier_input import CommitSpecifierInput as CommitSpecifierInput
from .payment_gate_config_v2 import PaymentGateConfigV2 as PaymentGateConfigV2
from .update_base_threshold_commit import UpdateBaseThresholdCommit as UpdateBaseThresholdCommit
from .spend_threshold_configuration import SpendThresholdConfiguration as SpendThresholdConfiguration
from .commit_hierarchy_configuration import CommitHierarchyConfiguration as CommitHierarchyConfiguration
from .spend_threshold_configuration_v2 import SpendThresholdConfigurationV2 as SpendThresholdConfigurationV2
from .prepaid_balance_threshold_configuration import (
    PrepaidBalanceThresholdConfiguration as PrepaidBalanceThresholdConfiguration,
)
from .prepaid_balance_threshold_configuration_v2 import (
    PrepaidBalanceThresholdConfigurationV2 as PrepaidBalanceThresholdConfigurationV2,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from ..._models import BaseModel
from .customers.invoice import Invoice

__all__ = ["ContractScheduleProServicesInvoiceResponse"]


class ContractScheduleProServicesInvoiceResponse(BaseModel):
    data: List[Invoice]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel
from .customer_detail import CustomerDetail

__all__ = ["CustomerRetrieveResponse"]


class CustomerRetrieveResponse(BaseModel):
    data: CustomerDetail



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.v1 import setting_upsert_avalara_credentials_params
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._base_client import make_request_options
from .billing_providers import (
    BillingProvidersResource,
    AsyncBillingProvidersResource,
    BillingProvidersResourceWithRawResponse,
    AsyncBillingProvidersResourceWithRawResponse,
    BillingProvidersResourceWithStreamingResponse,
    AsyncBillingProvidersResourceWithStreamingResponse,
)
from ....types.v1.setting_upsert_avalara_credentials_response import SettingUpsertAvalaraCredentialsResponse

__all__ = ["SettingsResource", "AsyncSettingsResource"]


class SettingsResource(SyncAPIResource):
    @cached_property
    def billing_providers(self) -> BillingProvidersResource:
        return BillingProvidersResource(self._client)

    @cached_property
    def with_raw_response(self) -> SettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return SettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return SettingsResourceWithStreamingResponse(self)

    def upsert_avalara_credentials(
        self,
        *,
        avalara_environment: Literal["PRODUCTION", "SANDBOX"],
        avalara_password: str,
        avalara_username: str,
        delivery_method_ids: SequenceNotStr[str],
        commit_transactions: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingUpsertAvalaraCredentialsResponse:
        """
        Set the Avalara credentials for some specified `delivery_method_ids`, which can
        be found in the `/listConfiguredBillingProviders` response. This maps the
        Avalara credentials to the appropriate billing entity. These credentials are
        only used for PLG Invoicing today.

        Args:
          avalara_environment: The Avalara environment to use (SANDBOX or PRODUCTION).

          avalara_password: The password for the Avalara account.

          avalara_username: The username for the Avalara account.

          delivery_method_ids: The delivery method IDs of the billing provider configurations to update, can be
              found in the response of the `/listConfiguredBillingProviders` endpoint.

          commit_transactions: Commit transactions if you want Metronome tax calculations used for reporting
              and tax filings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/upsertAvalaraCredentials",
            body=maybe_transform(
                {
                    "avalara_environment": avalara_environment,
                    "avalara_password": avalara_password,
                    "avalara_username": avalara_username,
                    "delivery_method_ids": delivery_method_ids,
                    "commit_transactions": commit_transactions,
                },
                setting_upsert_avalara_credentials_params.SettingUpsertAvalaraCredentialsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingUpsertAvalaraCredentialsResponse,
        )


class AsyncSettingsResource(AsyncAPIResource):
    @cached_property
    def billing_providers(self) -> AsyncBillingProvidersResource:
        return AsyncBillingProvidersResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSettingsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSettingsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSettingsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncSettingsResourceWithStreamingResponse(self)

    async def upsert_avalara_credentials(
        self,
        *,
        avalara_environment: Literal["PRODUCTION", "SANDBOX"],
        avalara_password: str,
        avalara_username: str,
        delivery_method_ids: SequenceNotStr[str],
        commit_transactions: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SettingUpsertAvalaraCredentialsResponse:
        """
        Set the Avalara credentials for some specified `delivery_method_ids`, which can
        be found in the `/listConfiguredBillingProviders` response. This maps the
        Avalara credentials to the appropriate billing entity. These credentials are
        only used for PLG Invoicing today.

        Args:
          avalara_environment: The Avalara environment to use (SANDBOX or PRODUCTION).

          avalara_password: The password for the Avalara account.

          avalara_username: The username for the Avalara account.

          delivery_method_ids: The delivery method IDs of the billing provider configurations to update, can be
              found in the response of the `/listConfiguredBillingProviders` endpoint.

          commit_transactions: Commit transactions if you want Metronome tax calculations used for reporting
              and tax filings.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/upsertAvalaraCredentials",
            body=await async_maybe_transform(
                {
                    "avalara_environment": avalara_environment,
                    "avalara_password": avalara_password,
                    "avalara_username": avalara_username,
                    "delivery_method_ids": delivery_method_ids,
                    "commit_transactions": commit_transactions,
                },
                setting_upsert_avalara_credentials_params.SettingUpsertAvalaraCredentialsParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SettingUpsertAvalaraCredentialsResponse,
        )


class SettingsResourceWithRawResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.upsert_avalara_credentials = to_raw_response_wrapper(
            settings.upsert_avalara_credentials,
        )

    @cached_property
    def billing_providers(self) -> BillingProvidersResourceWithRawResponse:
        return BillingProvidersResourceWithRawResponse(self._settings.billing_providers)


class AsyncSettingsResourceWithRawResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.upsert_avalara_credentials = async_to_raw_response_wrapper(
            settings.upsert_avalara_credentials,
        )

    @cached_property
    def billing_providers(self) -> AsyncBillingProvidersResourceWithRawResponse:
        return AsyncBillingProvidersResourceWithRawResponse(self._settings.billing_providers)


class SettingsResourceWithStreamingResponse:
    def __init__(self, settings: SettingsResource) -> None:
        self._settings = settings

        self.upsert_avalara_credentials = to_streamed_response_wrapper(
            settings.upsert_avalara_credentials,
        )

    @cached_property
    def billing_providers(self) -> BillingProvidersResourceWithStreamingResponse:
        return BillingProvidersResourceWithStreamingResponse(self._settings.billing_providers)


class AsyncSettingsResourceWithStreamingResponse:
    def __init__(self, settings: AsyncSettingsResource) -> None:
        self._settings = settings

        self.upsert_avalara_credentials = async_to_streamed_response_wrapper(
            settings.upsert_avalara_credentials,
        )

    @cached_property
    def billing_providers(self) -> AsyncBillingProvidersResourceWithStreamingResponse:
        return AsyncBillingProvidersResourceWithStreamingResponse(self._settings.billing_providers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["CreditLedgerEntry"]


class CreditLedgerEntry(BaseModel):
    amount: float
    """an amount representing the change to the customer's credit balance"""

    created_by: str

    credit_grant_id: str
    """the credit grant this entry is related to"""

    effective_at: datetime

    reason: str

    running_balance: float
    """
    the running balance for this credit type at the time of the ledger entry,
    including all preceding charges
    """

    invoice_id: Optional[str] = None
    """
    if this entry is a deduction, the Metronome ID of the invoice where the credit
    deduction was consumed; if this entry is a grant, the Metronome ID of the
    invoice where the grant's paid_amount was charged
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["CustomerSetBillingConfigurationsResponse", "Data"]


class Data(BaseModel):
    id: Optional[str] = None
    """ID of the created configuration"""

    billing_provider: Optional[
        Literal[
            "aws_marketplace",
            "stripe",
            "netsuite",
            "custom",
            "azure_marketplace",
            "quickbooks_online",
            "workday",
            "gcp_marketplace",
            "metronome",
        ]
    ] = None
    """The billing provider set for this configuration."""

    configuration: Optional[Dict[str, object]] = None
    """Configuration for the billing provider.

    The structure of this object is specific to the billing provider and delivery
    method combination.
    """

    customer_id: Optional[str] = None
    """ID of the customer this configuration is associated with."""

    delivery_method_id: Optional[str] = None
    """ID of the delivery method used for this customer configuration."""

    tax_provider: Optional[Literal["anrok", "avalara", "stripe"]] = None
    """The tax provider set for this configuration."""


class CustomerSetBillingConfigurationsResponse(BaseModel):
    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ...types.v1 import billable_metric_list_params, billable_metric_create_params, billable_metric_archive_params
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncCursorPage, AsyncCursorPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.shared_params.property_filter import PropertyFilter
from ...types.shared_params.event_type_filter import EventTypeFilter
from ...types.v1.billable_metric_list_response import BillableMetricListResponse
from ...types.v1.billable_metric_create_response import BillableMetricCreateResponse
from ...types.v1.billable_metric_archive_response import BillableMetricArchiveResponse
from ...types.v1.billable_metric_retrieve_response import BillableMetricRetrieveResponse

__all__ = ["BillableMetricsResource", "AsyncBillableMetricsResource"]


class BillableMetricsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BillableMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return BillableMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BillableMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return BillableMetricsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        name: str,
        aggregation_key: str | Omit = omit,
        aggregation_type: Literal["COUNT", "LATEST", "MAX", "SUM", "UNIQUE"] | Omit = omit,
        custom_fields: Dict[str, str] | Omit = omit,
        event_type_filter: EventTypeFilter | Omit = omit,
        group_keys: Iterable[SequenceNotStr[str]] | Omit = omit,
        property_filters: Iterable[PropertyFilter] | Omit = omit,
        sql: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillableMetricCreateResponse:
        """
        Create billable metrics programmatically with this endpoint—an essential step in
        configuring your pricing and packaging in Metronome.

        A billable metric is a customizable query that filters and aggregates events
        from your event stream. These metrics are continuously tracked as usage data
        enters Metronome through the ingestion pipeline. The ingestion process
        transforms raw usage data into actionable pricing metrics, enabling accurate
        metering and billing for your products.

        ### Use this endpoint to:

        - Create individual or multiple billable metrics as part of a setup workflow.
        - Automate the entire pricing configuration process, from metric creation to
          customer contract setup.
        - Define metrics using either standard filtering/aggregation or a custom SQL
          query.

        ### Key response fields:

        - The ID of the billable metric that was created
        - The created billable metric will be available to be used in Products, usage
          endpoints, and alerts.

        ### Usage guidelines:

        - Metrics defined using standard filtering and aggregation are Streaming
          billable metrics, which have been optimized for ultra low latency and high
          throughput workflows.
        - Use SQL billable metrics if you require more flexible aggregation options.

        Args:
          name: The display name of the billable metric.

          aggregation_key: Specifies the type of aggregation performed on matching events. Required if
              `sql` is not provided.

          aggregation_type: Specifies the type of aggregation performed on matching events.

          custom_fields: Custom fields to attach to the billable metric.

          event_type_filter: An optional filtering rule to match the 'event_type' property of an event.

          group_keys: Property names that are used to group usage costs on an invoice. Each entry
              represents a set of properties used to slice events into distinct buckets.

          property_filters: A list of filters to match events to this billable metric. Each filter defines a
              rule on an event property. All rules must pass for the event to match the
              billable metric.

          sql: The SQL query associated with the billable metric. This field is mutually
              exclusive with aggregation_type, event_type_filter, property_filters,
              aggregation_key, and group_keys. If provided, these other fields must be
              omitted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/billable-metrics/create",
            body=maybe_transform(
                {
                    "name": name,
                    "aggregation_key": aggregation_key,
                    "aggregation_type": aggregation_type,
                    "custom_fields": custom_fields,
                    "event_type_filter": event_type_filter,
                    "group_keys": group_keys,
                    "property_filters": property_filters,
                    "sql": sql,
                },
                billable_metric_create_params.BillableMetricCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillableMetricCreateResponse,
        )

    def retrieve(
        self,
        *,
        billable_metric_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillableMetricRetrieveResponse:
        """
        Retrieves the complete configuration for a specific billable metric by its ID.
        Use this to review billable metric setup before associating it with products.
        Returns the metric's `name`, `event_type_filter`, `property_filters`,
        `aggregation_type`, `aggregation_key`, `group_keys`, `custom fields`, and
        `SQL query` (if it's a SQL billable metric).

        Important:

        - Archived billable metrics will include an `archived_at` timestamp; they no
          longer process new usage events but remain accessible for historical
          reference.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not billable_metric_id:
            raise ValueError(f"Expected a non-empty value for `billable_metric_id` but received {billable_metric_id!r}")
        return self._get(
            f"/v1/billable-metrics/{billable_metric_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillableMetricRetrieveResponse,
        )

    def list(
        self,
        *,
        include_archived: bool | Omit = omit,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPage[BillableMetricListResponse]:
        """Retrieves all billable metrics with their complete configurations.

        Use this for
        programmatic discovery and management of billable metrics, such as associating
        metrics to products and auditing for orphaned or archived metrics. Important:
        Archived metrics are excluded by default; use `include_archived`=`true`
        parameter to include them.

        Args:
          include_archived: If true, the list of returned metrics will include archived metrics

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/billable-metrics",
            page=SyncCursorPage[BillableMetricListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_archived": include_archived,
                        "limit": limit,
                        "next_page": next_page,
                    },
                    billable_metric_list_params.BillableMetricListParams,
                ),
            ),
            model=BillableMetricListResponse,
        )

    def archive(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillableMetricArchiveResponse:
        """Use this endpoint to retire billable metrics that are no longer used.

        After a
        billable metric is archived, that billable metric can no longer be used in any
        new Products to define how that product should be metered. If you archive a
        billable metric that is already associated with a Product, the Product will
        continue to function as usual, metering based on the definition of the archived
        billable metric.

        Archived billable metrics will be returned on the `getBillableMetric` and
        `listBillableMetrics` endpoints with a populated `archived_at` field.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/billable-metrics/archive",
            body=maybe_transform({"id": id}, billable_metric_archive_params.BillableMetricArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillableMetricArchiveResponse,
        )


class AsyncBillableMetricsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBillableMetricsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBillableMetricsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBillableMetricsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/Metronome-Industries/metronome-python#with_streaming_response
        """
        return AsyncBillableMetricsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        name: str,
        aggregation_key: str | Omit = omit,
        aggregation_type: Literal["COUNT", "LATEST", "MAX", "SUM", "UNIQUE"] | Omit = omit,
        custom_fields: Dict[str, str] | Omit = omit,
        event_type_filter: EventTypeFilter | Omit = omit,
        group_keys: Iterable[SequenceNotStr[str]] | Omit = omit,
        property_filters: Iterable[PropertyFilter] | Omit = omit,
        sql: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillableMetricCreateResponse:
        """
        Create billable metrics programmatically with this endpoint—an essential step in
        configuring your pricing and packaging in Metronome.

        A billable metric is a customizable query that filters and aggregates events
        from your event stream. These metrics are continuously tracked as usage data
        enters Metronome through the ingestion pipeline. The ingestion process
        transforms raw usage data into actionable pricing metrics, enabling accurate
        metering and billing for your products.

        ### Use this endpoint to:

        - Create individual or multiple billable metrics as part of a setup workflow.
        - Automate the entire pricing configuration process, from metric creation to
          customer contract setup.
        - Define metrics using either standard filtering/aggregation or a custom SQL
          query.

        ### Key response fields:

        - The ID of the billable metric that was created
        - The created billable metric will be available to be used in Products, usage
          endpoints, and alerts.

        ### Usage guidelines:

        - Metrics defined using standard filtering and aggregation are Streaming
          billable metrics, which have been optimized for ultra low latency and high
          throughput workflows.
        - Use SQL billable metrics if you require more flexible aggregation options.

        Args:
          name: The display name of the billable metric.

          aggregation_key: Specifies the type of aggregation performed on matching events. Required if
              `sql` is not provided.

          aggregation_type: Specifies the type of aggregation performed on matching events.

          custom_fields: Custom fields to attach to the billable metric.

          event_type_filter: An optional filtering rule to match the 'event_type' property of an event.

          group_keys: Property names that are used to group usage costs on an invoice. Each entry
              represents a set of properties used to slice events into distinct buckets.

          property_filters: A list of filters to match events to this billable metric. Each filter defines a
              rule on an event property. All rules must pass for the event to match the
              billable metric.

          sql: The SQL query associated with the billable metric. This field is mutually
              exclusive with aggregation_type, event_type_filter, property_filters,
              aggregation_key, and group_keys. If provided, these other fields must be
              omitted.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/billable-metrics/create",
            body=await async_maybe_transform(
                {
                    "name": name,
                    "aggregation_key": aggregation_key,
                    "aggregation_type": aggregation_type,
                    "custom_fields": custom_fields,
                    "event_type_filter": event_type_filter,
                    "group_keys": group_keys,
                    "property_filters": property_filters,
                    "sql": sql,
                },
                billable_metric_create_params.BillableMetricCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillableMetricCreateResponse,
        )

    async def retrieve(
        self,
        *,
        billable_metric_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillableMetricRetrieveResponse:
        """
        Retrieves the complete configuration for a specific billable metric by its ID.
        Use this to review billable metric setup before associating it with products.
        Returns the metric's `name`, `event_type_filter`, `property_filters`,
        `aggregation_type`, `aggregation_key`, `group_keys`, `custom fields`, and
        `SQL query` (if it's a SQL billable metric).

        Important:

        - Archived billable metrics will include an `archived_at` timestamp; they no
          longer process new usage events but remain accessible for historical
          reference.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not billable_metric_id:
            raise ValueError(f"Expected a non-empty value for `billable_metric_id` but received {billable_metric_id!r}")
        return await self._get(
            f"/v1/billable-metrics/{billable_metric_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillableMetricRetrieveResponse,
        )

    def list(
        self,
        *,
        include_archived: bool | Omit = omit,
        limit: int | Omit = omit,
        next_page: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[BillableMetricListResponse, AsyncCursorPage[BillableMetricListResponse]]:
        """Retrieves all billable metrics with their complete configurations.

        Use this for
        programmatic discovery and management of billable metrics, such as associating
        metrics to products and auditing for orphaned or archived metrics. Important:
        Archived metrics are excluded by default; use `include_archived`=`true`
        parameter to include them.

        Args:
          include_archived: If true, the list of returned metrics will include archived metrics

          limit: Max number of results that should be returned

          next_page: Cursor that indicates where the next page of results should start.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/v1/billable-metrics",
            page=AsyncCursorPage[BillableMetricListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "include_archived": include_archived,
                        "limit": limit,
                        "next_page": next_page,
                    },
                    billable_metric_list_params.BillableMetricListParams,
                ),
            ),
            model=BillableMetricListResponse,
        )

    async def archive(
        self,
        *,
        id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BillableMetricArchiveResponse:
        """Use this endpoint to retire billable metrics that are no longer used.

        After a
        billable metric is archived, that billable metric can no longer be used in any
        new Products to define how that product should be metered. If you archive a
        billable metric that is already associated with a Product, the Product will
        continue to function as usual, metering based on the definition of the archived
        billable metric.

        Archived billable metrics will be returned on the `getBillableMetric` and
        `listBillableMetrics` endpoints with a populated `archived_at` field.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/billable-metrics/archive",
            body=await async_maybe_transform({"id": id}, billable_metric_archive_params.BillableMetricArchiveParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=BillableMetricArchiveResponse,
        )


class BillableMetricsResourceWithRawResponse:
    def __init__(self, billable_metrics: BillableMetricsResource) -> None:
        self._billable_metrics = billable_metrics

        self.create = to_raw_response_wrapper(
            billable_metrics.create,
        )
        self.retrieve = to_raw_response_wrapper(
            billable_metrics.retrieve,
        )
        self.list = to_raw_response_wrapper(
            billable_metrics.list,
        )
        self.archive = to_raw_response_wrapper(
            billable_metrics.archive,
        )


class AsyncBillableMetricsResourceWithRawResponse:
    def __init__(self, billable_metrics: AsyncBillableMetricsResource) -> None:
        self._billable_metrics = billable_metrics

        self.create = async_to_raw_response_wrapper(
            billable_metrics.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            billable_metrics.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            billable_metrics.list,
        )
        self.archive = async_to_raw_response_wrapper(
            billable_metrics.archive,
        )


class BillableMetricsResourceWithStreamingResponse:
    def __init__(self, billable_metrics: BillableMetricsResource) -> None:
        self._billable_metrics = billable_metrics

        self.create = to_streamed_response_wrapper(
            billable_metrics.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            billable_metrics.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            billable_metrics.list,
        )
        self.archive = to_streamed_response_wrapper(
            billable_metrics.archive,
        )


class AsyncBillableMetricsResourceWithStreamingResponse:
    def __init__(self, billable_metrics: AsyncBillableMetricsResource) -> None:
        self._billable_metrics = billable_metrics

        self.create = async_to_streamed_response_wrapper(
            billable_metrics.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            billable_metrics.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            billable_metrics.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            billable_metrics.archive,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, Required, TypedDict

__all__ = ["AlertListParams"]


class AlertListParams(TypedDict, total=False):
    customer_id: Required[str]
    """The Metronome ID of the customer"""

    next_page: str
    """Cursor that indicates where the next page of results should start."""

    alert_statuses: List[Literal["ENABLED", "DISABLED", "ARCHIVED"]]
    """Optionally filter by threshold notification status.

    If absent, only enabled notifications will be returned.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import Literal

from .tier import Tier
from ..._models import BaseModel
from .credit_type_data import CreditTypeData

__all__ = ["Rate"]


class Rate(BaseModel):
    rate_type: Literal["FLAT", "PERCENTAGE", "SUBSCRIPTION", "CUSTOM", "TIERED"]

    credit_type: Optional[CreditTypeData] = None

    custom_rate: Optional[Dict[str, object]] = None
    """Only set for CUSTOM rate_type.

    This field is interpreted by custom rate processors.
    """

    is_prorated: Optional[bool] = None
    """Default proration configuration.

    Only valid for SUBSCRIPTION rate_type. Must be set to true.
    """

    price: Optional[float] = None
    """Default price.

    For FLAT rate_type, this must be >=0. For PERCENTAGE rate_type, this is a
    decimal fraction, e.g. use 0.1 for 10%; this must be >=0 and <=1.
    """

    pricing_group_values: Optional[Dict[str, str]] = None
    """
    if pricing groups are used, this will contain the values used to calculate the
    price
    """

    quantity: Optional[float] = None
    """Default quantity. For SUBSCRIPTION rate_type, this must be >=0."""

    tiers: Optional[List[Tier]] = None
    """Only set for TIERED rate_type."""

    use_list_prices: Optional[bool] = None
    """Only set for PERCENTAGE rate_type.

    Defaults to false. If true, rate is computed using list prices rather than the
    standard rates for this product on the contract.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["AlertArchiveParams"]


class AlertArchiveParams(TypedDict, total=False):
    id: Required[str]
    """The Metronome ID of the threshold notification"""

    release_uniqueness_key: bool
    """
    If true, resets the uniqueness key on this threshold notification so it can be
    re-used
    """
